//+------------------------------------------------------------------+
//|                                          newMACD_Color_Alert.mq4 |
//|                                Copyright  2005, David W. Thomas |
//|                                           mailto:davidwt@usa.net |
//+------------------------------------------------------------------+
// This is the correct computation and display of MACD.
#property copyright "Copyright  2005, David W. Thomas"
#property link      "mailto:davidwt@usa.net"

#property indicator_separate_window
#property indicator_buffers 5
#property indicator_color1 Blue
#property indicator_color2 Red
#property indicator_color4 Green
#property indicator_color5 Red
#property indicator_level1 0.0
#property indicator_width4 2
#property indicator_width5 2

//---- input parameters
extern int FastMAPeriod = 12;
extern int SlowMAPeriod = 26;
extern int SignalMAPeriod = 9;
extern bool isAlert=true;
extern bool isArrows=true;


//---- buffers
double MACDLineBuffer[];
double SignalLineBuffer[];
double HistogramBuffer[];
double HistUpBuffer[];
double HistDnBuffer[];
//---- variables
double alpha = 0;
double alpha_1 = 0;




void SetArrow(datetime ArrowTime, double Price, double ArrowCode, color ArrowColor)
{
 int err;
 string ArrowName = DoubleToStr(ArrowTime,0);
   if (ObjectFind(ArrowName) != -1) ObjectDelete(ArrowName);
   if(!ObjectCreate(ArrowName, OBJ_ARROW, 0, ArrowTime, Price))
    {
      err=GetLastError();
      Print("error: can't create Arrow! code #",err);
      return;
    }
   else
   {
     ObjectSet(ArrowName, OBJPROP_ARROWCODE, ArrowCode);
     ObjectSet(ArrowName, OBJPROP_COLOR , ArrowColor);
     ObjectsRedraw();
   }
}

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
   IndicatorDigits(Digits + 1);
   //---- indicators
   SetIndexStyle(0, DRAW_LINE);
   SetIndexBuffer(0, MACDLineBuffer);
   SetIndexDrawBegin(0, SlowMAPeriod);
   SetIndexStyle(1, DRAW_LINE, STYLE_DOT);
   SetIndexBuffer(1, SignalLineBuffer);
   SetIndexDrawBegin(1, SlowMAPeriod + SignalMAPeriod);
   SetIndexStyle(2, DRAW_NONE);
   SetIndexBuffer(2, HistogramBuffer);
   SetIndexDrawBegin(2, SlowMAPeriod + SignalMAPeriod);
   SetIndexStyle(3, DRAW_HISTOGRAM);
   SetIndexBuffer(3, HistUpBuffer);
   SetIndexDrawBegin(3, SlowMAPeriod + SignalMAPeriod);
   SetIndexStyle(4, DRAW_HISTOGRAM);
   SetIndexBuffer(4, HistDnBuffer);
   SetIndexDrawBegin(4, SlowMAPeriod + SignalMAPeriod);
   //---- name for DataWindow and indicator subwindow label
   IndicatorShortName("newMACD_Color_Alert(" + FastMAPeriod+"," + SlowMAPeriod + "," + SignalMAPeriod + ")");
   SetIndexLabel(0, "NMACD");
   SetIndexLabel(1, "Signal");
   SetIndexLabel(2, "Histogr");   
   //----
	  alpha = 2.0 / (SignalMAPeriod + 1.0);
	  alpha_1 = 1.0 - alpha;
   //----
   return(0);
}
//+------------------------------------------------------------------+
//| Custor indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
   //----
   ObjectsDeleteAll(0);
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
  {
   int limit;
   int counted_bars = IndicatorCounted();
   //---- check for possible errors
   if(counted_bars < 0) 
       return(-1);
   //---- last counted bar will be recounted
   if(counted_bars > 0) 
       counted_bars--;
   limit = Bars - counted_bars;
//----
   for(int i = limit; i >= 0; i--)
     {
       MACDLineBuffer[i] = iMA(NULL, 0, FastMAPeriod, 0, MODE_EMA, PRICE_CLOSE, i) - 
                           iMA(NULL, 0, SlowMAPeriod, 0, MODE_EMA, PRICE_CLOSE, i);
       SignalLineBuffer[i] = alpha*MACDLineBuffer[i] + alpha_1*SignalLineBuffer[i+1];
       HistogramBuffer[i] = MACDLineBuffer[i] - SignalLineBuffer[i];
       if(HistogramBuffer[i]>=0)HistUpBuffer[i]=HistogramBuffer[i];
       if(HistogramBuffer[i]<0)HistDnBuffer[i]=HistogramBuffer[i];
       if((HistogramBuffer[i]>0)&&(HistogramBuffer[i+1]<=0))
       {
         if (isArrows)SetArrow(Time[i],Low[i]-15*Point,241,Aqua);
         if(isAlert){
         static int lt1=0;
            if(lt1!=Time[0]){
               lt1=Time[0]; Alert("newMACD_Color_Alert, ","BUY, ",Symbol(),", ",Period());
       }
       }
       }
       if((HistogramBuffer[i]<0)&&(HistogramBuffer[i+1]>=0))
       {
         if (isArrows)SetArrow(Time[i],High[i]+15*Point,242,Gold);
         if(isAlert) {
         static int lt2=0;
            if(lt2!=Time[0]){
               lt2=Time[0];Alert("newMACD_Color_Alert, ","SELL, ",Symbol(),", ",Period());
       }
       }
       }
    } 
//----
   return(0);
  }
//+------------------------------------------------------------------+